<?php
/*
Plugin Name: Facebook Shared Comments
Description: This plugin enables easy cross-posting of entries to Facebook, while also enabling comments to show up both on the wordpress blog and Facebook while remaining syncronized.
Author: Michael Stewart
Version: 1.0

Copyright (c) 2009 Michael Stewart

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

define('FBSC_DIR', 'fb-shared-comments');

function fbsc_admin_menu() {
	add_options_page('Facebook Shared Comments', 'FB Comments', 'manage_options', 'fbsc_options_page', 'fbsc_options_page');
	add_meta_box('fbsc', 'Facebook Shared Comments', 'fbsc_meta_box', 'post', 'normal', 'low');
}


function fbsc_options_page() {
	require_once('fbsc-options.php');
}


function fbsc_meta_box() { 
	require_once('fbsc-meta_box.php');
}


function fbsc_parse_request($wp) {
	if (array_key_exists('fbsc', $wp->query_vars)) {
		$allowed = array('canvas', 'xd');
		if (in_array($wp->query_vars['fbsc'], $allowed)) {
			include($wp->query_vars['fbsc'].'.php');
		}
		exit;
	}
}


function fbsc_admin_xml_ns() {
	global $pagenow;

	if (in_array($pagenow, array('post.php','post-new.php'))) {
		echo ' xmlns:fb="http://www.facebook.com/2008/fbml" ';
	}
}


function fbsc_save_post($post_id) {

	if (!wp_verify_nonce($_POST['fbsc_nonce'], 'fbsc_post')) {
		return $post_id;
	} 

	// check to see if this has already been posted to facebook, if so there is nothing that can be done
	if (get_post_meta($post_id, '_fbsc_status', true) == 'publish') {
		return $post_id;
	}

	$note = get_post_meta($post_id, '_fbsc_note', true);
	$image = get_post_meta($post_id, '_fbsc_image', true);
	$uuid = get_post_meta($post_id, '_fbsc_uuid', true);
	$status = get_post_meta($post_id, '_fbsc_status', true);

	if (isset($_POST['fbsc_note'])) {
		if ($_POST['fbsc_note'] != $note) {
			$note = $_POST['fbsc_note'];
			update_post_meta($post_id, '_fbsc_note', $note);
		}
	}

	if ($uuid == '') {
		$uuid = fbsc_uuid();
		update_post_meta($post_id, '_fbsc_uuid', $uuid);
	}

	if (get_post_meta($post_id, '_fbsc_error', true) != '') {
		delete_post_meta($post_id, '_fbsc_error');
	}

	if (isset($_POST['fbsc_image'])) {
		if ($_POST['fbsc_image'] != $image) {
			$image = $_POST['fbsc_image'];
			if (fbsc_verify_image_url($image)) {
				update_post_meta($post_id, '_fbsc_image', $image);
			} else {
				update_post_meta($post_id, '_fbsc_status', 'image-error');
				update_post_meta($post_id, '_fbsc_error', 'Not a valid image: '.$image);
				return $post_id;
			}
		}
	}

	if (isset($_POST['fbsc_dopost']) && ($_POST['fbsc_dopost'] == 'yes')) {
		if (get_post_status($post_id) == 'publish') {
			update_post_meta($post_id, '_fbsc_status', 'posting');
			fbsc_post($post_id);
			return $post_id;
		} else {
			update_post_meta($post_id, '_fbsc_status', 'post-ready');
			return $post_id;
		}
	} else {
		update_post_meta($post_id, '_fbsc_status', 'draft');
		return $post_id;
	}

	return $post_id;
}


function fbsc_facebook_comments($hide) {
	global $post;

	if ( (get_post_status($post->ID) == 'publish') && (get_post_meta($post->ID, '_fbsc_status', true) == 'publish') && (get_post_meta($post->ID, '_fbsc_uuid') != '') ) {
		$uuid = get_post_meta($post->ID, '_fbsc_uuid', true);
		require_once('fbsc-comments.php');
		return true;
	}

	return $hide;

}


function fbsc_filter_query_vars($vars) {
	$vars[] = 'fbsc';
	return $vars;
}


function fbsc_fb() {

	if (!class_exists('Facebook')) {
		require_once('facebook-client/facebook.php');
	}
	static $fb = null;
	if ($fb === null) {
		$fb = new Facebook(get_option('fbsc_api_key'), get_option('fbsc_secret'), true);
	}

	return $fb;
}


function fbsc_post($post_id) {

	$note = get_post_meta($post_id, '_fbsc_note', true);
	$image = get_post_meta($post_id, '_fbsc_image', true);
	$uuid = get_post_meta($post_id, '_fbsc_uuid', true);

	$post = get_post($post_id);
	$permalink = get_permalink($post_id);

	$text = $post->post_excerpt;

	// generate excerpt if needed - based on wp_trim_excerpt (wp-includes/formatting.php)
	if ($text == '') {
		$text = $post->post_content;
		$text = strip_shortcodes($text);
		$text = apply_filters('the_content', $text);
		$text = strip_tags($text);
		$words = explode(' ', $text, 25);
		if (count($words) > 24) {
			array_push($words, '[...]');
			$text = implode(' ', $words);
		}
	}

	$attachment = array(
		'comments_xid' => $uuid,
		'name' => $post->post_title,
		'href' => $permalink,
		'caption' => '{*actor*} has made a new entry on '.get_bloginfo('name').':',
		'description' => $text
	);

	if ($image != '') {
		$attachment['media'] = array(
			array(
				'type' => 'image',
				'src' => $image,
				'href' => $permalink
			)
		);
	}

	$links = array(
		array(
			'text' => 'Read Full Entry',
			'href' => $permalink
		)
	);	

	try {
		$id = fbsc_fb()->api_client->stream_publish($note, $attachment, $links);
	} catch (FacebookRestClientException $e) {
		update_post_meta($post_id, '_fbsc_status', 'post-error');
		update_post_meta($post_id, '_fbsc_error', $e->getMessage());
		return;
	}
	
	update_post_meta($post_id, '_fbsc_status', 'publish');
	update_post_meta($post_id, '_fbsc_id', $id);

}

function fbsc_uuid() {
	$random = '';
	for ($i=0; $i<16; $i++) {
		$random .= chr(mt_rand(0,255));
	}

	$time_low = bin2hex(substr($random, 0, 4));
	$time_mid = bin2hex(substr($random, 4, 2));
	$time_hi_and_version = bin2hex(substr($random, 6, 2));
	$clock_seq_and_reserved = bin2hex(substr($random, 8, 2));
	$node = bin2hex(substr($random, 10, 6));

	$time_hi_and_version = hexdec($time_hi_and_version);
	$time_hi_and_version = $time_hi_and_version >> 4;
	$time_hi_and_version = $time_hi_and_version | 0x4000;

	$clock_seq_hi_and_reserved = hexdec($clock_seq_hi_and_reserved);
	$clock_seq_hi_and_reserved = $clock_seq_hi_and_reserved >> 2;
	$clock_seq_hi_and_reserved = $clock_seq_hi_and_reserved | 0x8000;
      
	return sprintf('%08s-%04s-%04x-%04x-%012s',
		$time_low, $time_mid, $time_hi_and_version, $clock_seq_hi_and_reserved, $node);

}

function fbsc_verify_image_url($url) {
	
	$http = new WP_Http;
	$ret = $http->head($url);

	if(!isset($ret['headers']['content-type'])) {
		return false;
	}
	$ct = $ret['headers']['content-type'];
	switch ($ct) {
		case 'image/jpeg':
		case 'image/gif':
		case 'image/png':
			return true;
	}
	return false;

}

add_action('admin_menu', 'fbsc_admin_menu');
add_action('parse_request', 'fbsc_parse_request');
add_action('admin_xml_ns', 'fbsc_admin_xml_ns');
add_action('save_post', 'fbsc_save_post');
add_filter('facebook_comments', 'fbsc_facebook_comments');
add_filter('query_vars', 'fbsc_filter_query_vars');


